@@ -3,12 +3,12 @@
 #  ==========
 #' @title Default Control Settings
 #' @description A auxiliary function for defining the control variables.
-#' @param loss a character string giving the name of the loss function 
+#' @param loss a character string giving the name of the loss function
 #'             allowed values are \code{'softmax'}, \code{'hs'} and \code{'ns'}.
 #' @param learning_rate a numeric giving the learning rate, the default value is \code{0.05}.
 #' @param learn_update an integer giving after how many tokens the learning rate
 #'                     should be updated. The default value is \code{100L}, which
-#'                     means the learning rate is updated every 100 tokens. 
+#'                     means the learning rate is updated every 100 tokens.
 #' @param word_vec_size an integer giving the length (size) of the word vectors.
 #' @param window_size an integer giving the size of the context window.
 #' @param epoch an integer giving the number of epochs.
@@ -23,14 +23,14 @@
 #' @param threshold a numeric giving the sampling threshold.
 #' @param label a character string specifying the label prefix (default is \code{'__label__'}).
 #' @param verbose an integer giving the verbosity level, the default value
-#'                is \code{0L} and shouldn't be changed since Rcpp::Rcout 
+#'                is \code{0L} and shouldn't be changed since Rcpp::Rcout
 #'                cann't handle the traffic.
 #' @param pretrained_vectors a character string giving the file path
-#'                           to the pretrained word vectors which are used 
+#'                           to the pretrained word vectors which are used
 #'                           for the supervised learning.
 #' @param output a character string giving the output file path.
 #' @param save_output a logical (default is \code{FALSE})
-#' @param seed an integer 
+#' @param seed an integer
 #' @param qnorm  a logical (default is \code{FALSE})
 #' @param retrain a logical (default is \code{FALSE})
 #' @param qout a logical (default is \code{FALSE})
@@ -44,15 +44,15 @@
 #' @return a list with the control variables.
 #' @examples
 #' ft_control(learning_rate=0.1)
-ft_control <- function(loss = c("softmax", "hs", "ns"), 
-                       learning_rate=0.05, learn_update=100L, word_vec_size=100L, 
+ft_control <- function(loss = c("softmax", "hs", "ns"),
+                       learning_rate=0.05, learn_update=100L, word_vec_size=100L,
                        window_size=5L, epoch=5L, min_count=5L, min_count_label=0L,
                        neg=5L, max_len_ngram=1L, nbuckets=2000000L, min_ngram=3L,
-                       max_ngram=6L, nthreads=1L, threshold=1e-4, label="__label__", 
-                       verbose=0, pretrained_vectors="", output="", save_output=FALSE, 
-                       seed=0L, qnorm=FALSE, retrain=FALSE, qout=FALSE, cutoff=0L, 
-                       dsub=2L, autotune_validation_file="", autotune_metric="f1", 
-                       autotune_predictions=1L, autotune_duration=300L, 
+                       max_ngram=6L, nthreads=1L, threshold=1e-4, label="__label__",
+                       verbose=0, pretrained_vectors="", output="", save_output=FALSE,
+                       seed=0L, qnorm=FALSE, retrain=FALSE, qout=FALSE, cutoff=0L,
+                       dsub=2L, autotune_validation_file="", autotune_metric="f1",
+                       autotune_predictions=1L, autotune_duration=300L,
                        autotune_model_size="") {
     loss <- match.arg(loss)
     cntrl <- as.list(environment())
@@ -62,20 +62,20 @@ ft_control <- function(loss = c("softmax", "hs", "ns"),
 
 
 check_control_arguments <- function(control) {
-    control_arguments <- c("input", "output", "learning_rate", "learn_update", 
-        "word_vec_size", "window_size", "epoch", "min_count",  "min_count_label", 
-        "neg", "max_len_ngram", "nbuckets", "min_ngram", "max_ngram", "nthreads", 
+    control_arguments <- c("input", "output", "learning_rate", "learn_update",
+        "word_vec_size", "window_size", "epoch", "min_count",  "min_count_label",
+        "neg", "max_len_ngram", "nbuckets", "min_ngram", "max_ngram", "nthreads",
         "threshold", "label", "verbose", "pretrained_vectors", "save_output", "seed",
-        "qnorm", "retrain", "qout", "cutoff", "dsub", "autotune_validation_file", 
-        "autotune_metric", "autotune_predictions", "autotune_duration", 
+        "qnorm", "retrain", "qout", "cutoff", "dsub", "autotune_validation_file",
+        "autotune_metric", "autotune_predictions", "autotune_duration",
         "autotune_model_size")
     if ( !all(control_arguments %in% names(control)) ) {
         i <- which(!control_arguments %in% names(control))
         if ( length(i) == 1) {
             stop("control argument '", control_arguments[i], "' is missing")
         } else {
-            stop("control arguments ", 
-                 paste(shQuote(control_arguments[i]), collapse=", "), 
+            stop("control arguments ",
+                 paste(shQuote(control_arguments[i]), collapse=", "),
                  " are missing")
         }
     }
@@ -119,15 +119,15 @@ fasttext <- function() {
     model$save <- function(file, what = c("model", "vectors", "output")) {
         ft_save(parent.env(environment())$model, file, what = what)
     }
-    
-    model$train <- function(file, method = c("supervised", "cbow", "skipgram"), 
+
+    model$train <- function(file, method = c("supervised", "cbow", "skipgram"),
                             control = ft_control(), ...) {
         self <- parent.env(environment())$model
         update_model(self, ft_train(file, method, control, ...))
         return(invisible(NULL))
     }
 
-    model$predict <- function(newdata, k = 1L, threshold = 0, 
+    model$predict <- function(newdata, k = 1L, threshold = 0,
                               rval = c("sparse", "dense", "slam"), ...) {
         self <- parent.env(environment())$model
         ft_predict(self, newdata, k, threshold, rval, ...)
@@ -145,6 +145,10 @@ fasttext <- function() {
         ft_word_vectors(parent.env(environment())$model, words)
     }
 
+    model$sentence_vectors <- function(sentences) {
+        ft_sentence_vectors(parent.env(environment())$model, sentences)
+    }
+
     model$nearest_neighbors <- function(word, k = 10L) {
         ft_nearest_neighbors(parent.env(environment())$model, word, k)
     }
@@ -154,7 +158,7 @@ fasttext <- function() {
     }
 
     class(model) <- "fasttext"
-    
+
     model
 }
 
@@ -173,10 +177,10 @@ fasttext <- function() {
 #  ft_train
 #  ========
 #' @title Train a Model
-#' @description Train a new word representation model or supervised 
+#' @description Train a new word representation model or supervised
 #'              classification model.
 #' @param file a character string giving the location of the input file.
-#' @param method a character string giving the method, possible values are 
+#' @param method a character string giving the method, possible values are
 #'               \code{'supervised'}, \code{'cbow'} and \code{'skipgram'}.
 #' @param control a list giving the control variables, for more information
 #'                see \code{\link{ft_control}}.
@@ -186,7 +190,7 @@ fasttext <- function() {
 #' cntrl <- ft_control(nthreads = 1L)
 #' model <- ft_train("my_data.txt", method="supervised", control = cntrl)
 #' }
-ft_train <- function(file, method = c("supervised", "cbow", "skipgram"), 
+ft_train <- function(file, method = c("supervised", "cbow", "skipgram"),
                      control = ft_control(), ...) {
     method <- match.arg(method)
     stopifnot( is.character(file), file.exists(file), inherits(control, "ft_control") )
@@ -196,7 +200,7 @@ ft_train <- function(file, method = c("supervised", "cbow", "skipgram"),
     control$method <- method
 
     check_control_arguments(control)
-    
+
     model <- fasttext()
     model$pointer <- Rft_train(control)
     .wrap_model(model)
@@ -215,18 +219,18 @@ ft_train <- function(file, method = c("supervised", "cbow", "skipgram"),
 #'                  on the probabilities. Predictions with probabilities
 #'                  below this lower bound are not returned. The default
 #'                  is \code{0} which means all predictions are returned.
-#' @param rval a character string controlling the return value, allowed 
+#' @param rval a character string controlling the return value, allowed
 #'      values are \code{"sparse"}, \code{"dense"} and \code{"slam"}.
 #'      The default is sparse, here the values are returned as a \code{data.frame}
 #'      in a format similar to a simple triplet matrix (sometimes refereed to as the
 #'      coordinate format). If \code{rval} is set to \code{"dense"}, a matrix
-#'      of the probabilities is returned. Similarly if \code{rval} is set to 
-#'      \code{"slam"}, a matrix in the simple triplet sparse format from the 
+#'      of the probabilities is returned. Similarly if \code{rval} is set to
+#'      \code{"slam"}, a matrix in the simple triplet sparse format from the
 #'      \pkg{slam} package is returned.
-#' @param ... currently not used. 
-#' @return \code{NULL} if a \code{'result_file'} is given otherwise 
-#'         if \code{'prob'} is true a \code{data.frame} with the predicted labels 
-#'         and the corresponding probabilities, if \code{'prob'} is false a 
+#' @param ... currently not used.
+#' @return \code{NULL} if a \code{'result_file'} is given otherwise
+#'         if \code{'prob'} is true a \code{data.frame} with the predicted labels
+#'         and the corresponding probabilities, if \code{'prob'} is false a
 #'         character vector with the predicted labels.
 #' @examples
 #' \dontrun{
@@ -234,7 +238,7 @@ ft_train <- function(file, method = c("supervised", "cbow", "skipgram"),
 #' }
 #' @name predict.supervised_model
 #' @rdname predict.supervised_model
-ft_predict <- function(model, newdata, k = 1L, threshold = 0, 
+ft_predict <- function(model, newdata, k = 1L, threshold = 0,
                        rval = c("sparse", "dense", "slam"), ...) {
     stopifnot( inherits(model, "fasttext"), inherits(newdata, "character") )
 
@@ -246,7 +250,7 @@ ft_predict <- function(model, newdata, k = 1L, threshold = 0,
     labels <- unique(pred$label)
     labels <- labels[order(as.integer(gsub("\\D", "", labels)))]
     j <- match(pred$label, labels)
-    probs <- simple_triplet_matrix(i = pred$id, j = j, v = pred$prob) 
+    probs <- simple_triplet_matrix(i = pred$id, j = j, v = pred$prob)
     colnames(probs) <- labels
     if (rval == "dense") as.matrix(probs) else probs
 }
@@ -363,6 +367,28 @@ ft_word_vectors <- function(model, words) {
 }
 
 
+# -----------------------------------------------------------
+#  get_sentence_vectors
+#  ================
+#' @title Get Sentence Vectors
+#' @description Obtain sentence vectors from a previously trained model.
+#' @param model an object inheriting from \code{"fasttext"}.
+#' @param sentences a character vector giving the sentences.
+#' @return a matrix containing the sentence vectors.
+#' @examples
+#' \dontrun{
+#' ft_sentence_vectors(model, c("sentence", "vector"))
+#' }
+ft_sentence_vectors <- function(model, sentences) {
+    stopifnot(is.character(sentences), all(nchar(sentences) > 0))
+    stopifnot( inherits(model, "fasttext") )
+    sentence_vector <- Rft_sentence_vectors(model$pointer, sentences)
+    sentence_vector <- do.call(rbind, sentence_vector)
+    rownames(sentence_vector) <- sentences
+    sentence_vector
+}
+
+
 # -----------------------------------------------------------
 #  ft_nearest_neighbors
 #  =================
@@ -398,7 +424,7 @@ ft_nearest_neighbors <- function(model, word, k = 10L) {
 ft_analogies <- function(model, word_triplets, k = 10L) {
     stopifnot(is.character(word_triplets), isTRUE(length(word_triplets) == 3L))
     stopifnot( inherits(model, "fasttext") )
-    Rft_analogies(model$pointer, word_triplets[1L], word_triplets[2L], 
+    Rft_analogies(model$pointer, word_triplets[1L], word_triplets[2L],
                   word_triplets[3L], as.integer(k))
 }
 
@@ -420,21 +446,21 @@ print.fasttext <- function(x, ...) {
 
 print.cbow_model <- function(x, ...) {
     cat("fastText", shQuote(x$model_type), "model:", "\n")
-    cat(sprintf("    %s tokens, %s words", 
+    cat(sprintf("    %s tokens, %s words",
         format(x$ntoken, big.mark=","),
         format(x$nwords, big.mark=",")), "\n")
 }
 
 print.skipgram_model <- function(x, ...) {
     cat("fastText", shQuote(x$model_type), "model:", "\n")
-    cat(sprintf("    %s tokens, %s words", 
+    cat(sprintf("    %s tokens, %s words",
         format(x$ntoken, big.mark=","),
         format(x$nwords, big.mark=",")), "\n")
 }
 
 print.supervised_model <- function(x, ...) {
     cat("fastText", shQuote(x$model_type), "model:", "\n")
-    cat(sprintf("    %s tokens, %s words, %s labels", 
+    cat(sprintf("    %s tokens, %s words, %s labels",
         format(x$ntoken, big.mark=","),
         format(x$nwords, big.mark=","),
         format(x$nlabels, big.mark=",")), "\n")
